-- fifo	upd485505
-- dd modified for registered write enable 
library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

entity fifo is
    generic ( wlength :integer := 5048; width : integer := 8);
    port (
		not_re : in std_logic;
		not_we : in std_logic;
		not_rstr,not_rstw : in std_logic;
		rck,wck : in std_logic;
		
		din : in std_logic_vector(width-1 downto 0);
		dout : inout std_logic_vector(width-1 downto 0)	
   );
end fifo;

library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;


architecture operate_fifo of fifo is

type mem_type is array (wlength-1 downto 0) of std_logic_vector(width-1 downto 0);

signal mem : mem_type;

signal inptr,outptr : std_logic_vector(12 downto 0);

signal not_we_reg : std_logic :='0'; -- clocked internal write enable

-- values for -35 parts
constant tAC : time := 25 ns;
constant tDS : time := 10 ns;
constant tDH : time := 3 ns;
constant tHZ : time := 25 ns;

begin




-- write clock setup and hold checks
assert1: for i in 0 to width-1 generate
x : ASSERT not((wck='1' and not wck'stable) and (not din(i)'stable(tDS))) 
        REPORT "setup violation"
        SEVERITY warning;

y : ASSERT not((wck='1' and din(i)'event) and (not wck'stable(tDH))) 
        REPORT "hold violation"
        SEVERITY warning;

end generate;

proc_input_reg:		  
process (not_rstw,wck,not_we,inptr,not_we_reg)
begin
  if rising_edge(wck) then
    not_we_reg <= not_we;
    if (not_rstw='0') then
      for i in inptr'reverse_range loop 
	    inptr(i) <= '0';
	  end loop;
    elsif (not_we_reg='0') then
      inptr <= inptr+"1";
    end if;
  end if;
end process proc_input_reg;

proc_input:
process (wck,not_we_reg,din,inptr)
begin
  if ((not_we_reg='0') and rising_edge(wck) and (not_rstw='1')) then
	mem(evec2int(inptr)) <= din;
  end if ;
end process proc_input;

proc_output_reg:		  
process (not_rstr,rck,not_re,outptr)
begin
  if rising_edge(rck) then
    if (not_rstr='0') then
      for j in outptr'reverse_range loop 
	    outptr(j) <= '0';
	  end loop;
    elsif (not_re='0') then
      outptr <= outptr+"1";
    end if;
  end if;
end process proc_output_reg;

proc_output:
process (rck,not_re,mem,outptr)
begin
  if rising_edge(rck) then
    if (not_re='0') then
	  dout <= mem(evec2int(outptr)) after tAC;
    else
	  for k in dout'reverse_range loop
	    dout(k) <= 'Z' after tHZ;
	  end loop;
	end if;
  end if ;
end process proc_output;

  
end operate_fifo;
