-- fifo	upd485505 w/ save/load frame
-- dd modified for registered write enable 
library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

entity fifop is
    generic ( wlength :integer := 5048; width : integer := 8
              );
    port (
-- these activate saveing and loading mem
		save_flag,load_flag : std_logic:='0'; -- 1 for save, 1 for load
			  save_fname,load_fname : string(1 to 12);  -- bypass port file names for save, load
              starta : integer :=0;
              cnt : integer := wlength;

		not_re : in std_logic;
		not_we : in std_logic;
		not_rstr,not_rstw : in std_logic;
		rck,wck : in std_logic;
		
		din : in std_logic_vector(width-1 downto 0);
		dout : inout std_logic_vector(width-1 downto 0)	
   );
end fifop;

library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;
use std.textio.all;
use work.io_stuff.all;


architecture operate_fifop of fifop is

type mem_type is array (wlength-1 downto 0) of std_logic_vector(width-1 downto 0);

signal mem : mem_type;

signal inptr,outptr : std_logic_vector(12 downto 0);

signal not_we_reg : std_logic :='0'; -- clocked internal write enable

-- values for -35 parts
constant tAC : time := 25 ns;
constant tDS : time := 10 ns;
constant tDH : time := 3 ns;
constant tHZ : time := 25 ns;
 
FILE load_handle : TEXT;
FILE save_handle : TEXT;


begin




-- write clock setup and hold checks
assert1: for i in 0 to width-1 generate
x : ASSERT not((wck='1' and not wck'stable) and (not din(i)'stable(tDS))) 
        REPORT "setup violation"
        SEVERITY warning;

y : ASSERT not((wck='1' and din(i)'event) and (not wck'stable(tDH))) 
        REPORT "hold violation"
        SEVERITY warning;

end generate;

proc_input_reg:		  
process (not_rstw,wck,not_we,inptr,not_we_reg)
begin
  if rising_edge(wck) then
    not_we_reg <= not_we;
    if (not_rstw='0') then
      for i in inptr'reverse_range loop 
	    inptr(i) <= '0';
	  end loop;
    elsif (not_we_reg='0') then
      inptr <= inptr+"1";
    end if;
  end if;
end process proc_input_reg;

proc_input:
process (wck,not_we_reg,din,inptr)
begin
  if ((not_we_reg='0') and rising_edge(wck) and (not_rstw='1')) then
	mem(evec2int(inptr)) <= din;
  end if ;
end process proc_input;

proc_output_reg:		  
process (not_rstr,rck,not_re,outptr)
begin
  if rising_edge(rck) then
    if (not_rstr='0') then
      for j in outptr'reverse_range loop 
	    outptr(j) <= '0';
	  end loop;
    elsif (not_re='0') then
      outptr <= outptr+"1";
    end if;
  end if;
end process proc_output_reg;

proc_output:
process (rck,not_re,mem,outptr)
begin
  if rising_edge(rck) then
    if (not_re='0') then
	  dout <= mem(evec2int(outptr)) after tAC;
    else
	  for k in dout'reverse_range loop
	    dout(k) <= 'Z' after tHZ;
	  end loop;
	end if;
  end if ;
end process proc_output;


proc_byp_in: process (load_flag)
variable datav,cntr : integer;
variable flag : boolean;
begin
	if (rising_edge(load_flag)) then
	cntr := 0;
    file_open(load_handle,load_fname,READ_MODE);
    for i in starta to wlength-1 loop
	    proc_vram_load(load_handle,datav,flag);
        mem(i) <= int2evec(datav)(width-1 downto 0);
		cntr := cntr+1;
		if (cntr=cnt) then exit; end if;
    end loop;
	file_close(load_handle);
	end if;
end process proc_byp_in;

proc_byp_out: process (save_flag)
variable datav,cntr : integer;
variable flag : boolean;
begin
	if (rising_edge(save_flag)) then
	cntr := 0;
    file_open(save_handle,save_fname,WRITE_MODE);
    for i in starta to wlength-1 loop
        datav := evec2int(mem(i));
	    proc_vram_save(save_handle,datav,width/4,flag);
		cntr := cntr+1;
		if (cntr=cnt) then exit; end if;
    end loop;
	file_close(save_handle);
	end if;
end process proc_byp_out;
	  

  
end operate_fifop;
